from datetime import timedelta
from prefect.utilities.tasks import defaults_from_attrs
from .query_service import Query


class QueryDatasysCompany(Query):
    def __init__(self, key, url=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        super().__init__(key, url, max_retries, retry_delay, **kwargs)
        self.name = "Datasys"
        self.log_prefix = self.build_log_prefix()

    @staticmethod
    def prep(datasys_id: dict, url: str):
        if "biid" not in datasys_id:
            return
        biid = datasys_id["biid"]
        return "{}/repository/company/{}?trace=false".format(url, biid)

    @defaults_from_attrs('url')
    def run(self, data, url=None, **kwargs):
        if "error" in data:
            return data
        full_url = self.prep(datasys_id=data, url=url)
        response = self.query(data=data, full_url=full_url)
        result = self.process(data=data, full_url=full_url, response=response)
        result = self.add_api_call(full_url=full_url, result=result)
        return result



import requests
import logging
from datetime import timedelta

from datamodelz.error import Error
from prefect import Task
from prefect.engine.result_handlers import JSONResultHandler
from prefect.utilities.tasks import defaults_from_attrs
from .query_service import Query
from . import errors


class QueryCA(Query):
    """
    Base Query.
    """

    def __init__(self, key, url=None, service=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        super().__init__(key, url, max_retries, retry_delay, **kwargs)
        self.name = "Query CA"
        self.log_prefix = self.build_log_prefix()
        self.service = service

    def prep(self, ca_id, url: str):
        return "{url}/entity/{ca_id}?includeUnverified=true".format(url=url, ca_id=ca_id)

    def query(self, data, full_url, **kwargs):
        if "error" in full_url:
            return full_url
        logging.debug("{}: url {}".format(self.log_prefix, full_url))
        try:
            response = requests.get(
                url=full_url,
                params={},
                headers={'Authorization': self.key},
            )
            if response is not None:
                return response
            return self.error(Error(check_name=self.log_prefix,
                                    value="Bad request",
                                    company=data,
                                    api_call=full_url))
        except Exception as err:
            return self.error(Error(check_name=self.log_prefix,
                                    value="Bad request: {}".format(err),
                                    company=data,
                                    api_call=full_url))

    @defaults_from_attrs('url', 'service')
    def run(self, data, url=None, service=None, **kwargs):
        full_url = self.prep(ca_id=data, url=url)
        response = self.query(data=data, full_url=full_url)
        result = self.process(data=data, full_url=full_url, response=response)
        result = self.add_api_call(full_url=full_url, result=result)
        return result


class QueryCACompany(QueryCA):
    """
    Query to get datasys or rankings id.
    """

    def __init__(self, key, url=None, service=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        super().__init__(key, url, max_retries, retry_delay, **kwargs)
        self.name = "Query CA Company"
        self.log_prefix = self.build_log_prefix()
        self.service = service

    def prep(self, ca_id, url: str):
        return "{url}/entity/{ca_id}?includeUnverified=true".format(url=url, ca_id=ca_id)

    def post_process(self, data, full_url, result, service):
        final = {}
        if "error" in result:
            return result
        if "biids" not in result:
            return self.error(Error(check_name=self.log_prefix,
                                    value=errors.service_name_dne("biids"),
                                    company=data,
                                    api_call=full_url))
        lst = result["biids"]
        for item in lst:
            if item["service"] == service:
                final["biid"] = item["value"]
                return final
        return self.error(Error(check_name=self.log_prefix,
                                value=errors.service_name_dne(service),
                                company=data,
                                api_call=full_url))

    @defaults_from_attrs('url', 'service')
    def run(self, data, url=None, service=None, **kwargs):
        full_url = self.prep(ca_id=data, url=url)
        response = self.query(data=data, full_url=full_url)
        result = self.process(data=data, full_url=full_url, response=response)
        result = self.post_process(data=data, full_url=full_url, result=result, service=service)
        result = self.add_api_call(full_url=full_url, result=result)
        return result



import requests
import logging
from prefect import task, Task
from prefect.utilities.tasks import defaults_from_attrs
from prefect.client import Secret
from prefect.utilities.notifications import slack_notifier
from prefect.engine.state import Failed
from datetime import timedelta
from . import errors
from datamodelz.error import Error


class Query(Task):
    def __init__(self, key, url=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        super().__init__(max_retries, retry_delay, **kwargs)
        self.name = "Query"
        self.key = key
        self.url = url
        self.log_prefix = self.build_log_prefix()

    def query(self, data, full_url, **kwargs):
        if "error" in full_url:
            return full_url
        logging.debug("{}: url {}".format(self.log_prefix, full_url))
        try:
            response = requests.get(
                url=full_url,
                params={},
                headers={'Authorization': self.key},
            )
            return response
        except Exception as err:
            return self.error(
                Error(check_name="Query", value="Bad request: {}".format(err), company=data, api_call=full_url))

    def process(self, data, full_url, response, **kwargs):
        print("queried {} with response {}".format(full_url, response))
        if "error" in response:
            return response
        try:
            if response.status_code != 200:
                return self.error(Error(check_name="Query", value=errors.response_code(response.status_code),
                                        company=data, api_call=full_url))
            doc = response.json()
            if doc is None or not doc or doc == {}:
                return self.error(Error(check_name="Query", value=errors.bad_response, company=data, api_call=full_url))
            return doc
        except Exception as err:
            return self.error(Error(check_name="Query", value="{}: {}".format(errors.bad_response, err), company=data,
                                    api_call=full_url))

    @staticmethod
    def add_api_call(full_url, result):
        result["apiCall"] = full_url
        return result

    def error(self, error) -> dict:
        full_msg = "{}: {}".format(self.log_prefix, error)
        logging.error(full_msg)
        return {"error": [error]}

    def build_log_prefix(self) -> str:
        return self.name.title().replace(' ', '')

    @defaults_from_attrs('url')
    def run(self, data, url=None, **kwargs):
        response = self.query(data, url)
        result = self.process(data, url, response)
        return result



import logging
from datetime import timedelta

import requests
from prefect.engine.result_handlers import JSONResultHandler
from prefect.utilities.tasks import defaults_from_attrs

from commontaskz import errors
from requests import Response

from .query_service import Query
from datamodelz.error import Error


class GenericTimeseries(Query):
    """
    Basic
    """
    def __init__(self, key, url=None, timeseries=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        super().__init__(key, url, max_retries, retry_delay, **kwargs)
        self.name = "Generic Timeseries"
        self.url = url
        self.timeseries = timeseries
        self.key = "apiKey {}".format(key)
        self.log_prefix = self.build_log_prefix()

    def query(self, data, full_url, timeseries="", **kwargs):
        if "error" in full_url:
            return full_url
        logging.debug("{}: url {}".format(self.log_prefix, full_url))
        try:
            response = requests.get(
                url=full_url,
                params={},
                headers={'Authorization': self.key},
            )
            return response
        except Exception as err:
            return self.error(
                Error(check_name="Query",
                      timeseries_code=timeseries,
                      value="Bad request: {}".format(err),
                      company=data,
                      api_call=full_url)
            )

    def process(self, data, full_url, response, timeseries=None, **kwargs):
        if "error" in response:
            return response
        try:
            if type(response) == str:
                return self.error(Error(check_name=self.log_prefix, value=response,
                                        timeseries_code=timeseries, company=data, api_call=full_url))
            if type(response) != Response:
                return self.error(Error(check_name=self.log_prefix, value="Bad Response",
                                        timeseries_code=timeseries, company=data, api_call=full_url))
            if response.status_code != 200:
                return self.error(Error(check_name=self.log_prefix, value=str(response.status_code),
                                        timeseries_code=timeseries, company=data, api_call=full_url))
            doc = response.json()
            if doc is None or not doc or doc == {}:
                return self.error(Error(check_name=self.log_prefix, value="Empty Response",
                                        timeseries_code=timeseries, company=data, api_call=full_url))
            return doc
        except Exception as err:
            return self.error(Error(check_name=self.log_prefix, value="Bad Response: {}".format(str(err)),
                                    timeseries_code=timeseries, company=data, api_call=full_url))


class QueryTimeseries(Query):
    def __init__(self, key, url=None, timeseries=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        super().__init__(key, url, timeseries, max_retries, retry_delay, **kwargs)
        self.name = "Query Timeseries"
        self.key = "apiKey {}".format(key)
        self.log_prefix = self.build_log_prefix()

    def prep(self, data: str, url: str, timeseries: str):
        if "error" in data:
            return data
        new_url = "{url}/timeseries/{timeseries}/{ca_id}" \
            .format(url=url, ca_id=data, timeseries=timeseries)
        return new_url

    @defaults_from_attrs('url', 'timeseries')
    def run(self, data, timeseries=None, url=None, **kwargs):
        full_url = self.prep(data=data, url=url, timeseries=timeseries)
        response = self.query(data=data, full_url=full_url, timeseries=timeseries)
        result = self.process(data=data, full_url=full_url, response=response, timeseries=timeseries)
        return result


class GetTimeseriesData(GenericTimeseries):
    def __init__(self, key, url=None, order=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        super().__init__(key, url, max_retries, retry_delay, **kwargs)
        self.name = "Get Timeseries Data"
        self.order = order
        self.key = "apiKey {}".format(key)
        self.log_prefix = self.build_log_prefix()

    @staticmethod
    def prep(data: str,
             url: str,
             timeseries: str,
             skip: int,
             limit: int,
             order: str,
             collapse: str,
             aggregate: str,
             transform: str):
        new_url = "{url}/timeseries/{timeseries}/{data}?skip={skip}&limit={limit}"\
            .format(url=url, data=data.lower(), timeseries=timeseries.lower(), skip=skip, limit=limit)
        if order:
            new_url += "&order={}".format(order.lower())
        if collapse:
            new_url += "&collapse={}".format(collapse.lower())
        if aggregate:
            new_url += "&aggregate={}".format(aggregate.lower())
        if transform:
            new_url += "&transform={}".format(transform.lower())
        return new_url

    @staticmethod
    def join(final, result):
        if "error" in result:
            return result
        if "data" in result:
            final["data"] = final["data"] + result["data"]
        return final

    def done(self, final, result, skip):
        # make data dict
        if "data" not in final:
            final = result
        else:
            final = self.join(final, result)
        if "error" in result or skip >= result["totalCount"]:  # stop because bad response or done getting results
            return final, True
        return final, False  # still more to go and no errors

    @defaults_from_attrs('url', 'order')
    def run(self, data, timeseries=None, url=None, order=None, collapse=None, aggregate=None, transform=None, **kwargs):
        """
        Repeats API call to get all data via paging for given timeseries type & company id
        :param aggregate:
        :param order:
        :param data:
        :param timeseries:
        :param url:
        :param collapse:
        :param aggregation:
        :param transform:
        :return: will return error or final results
        """
        if "error" in data:
            return data
        final = {}
        skip, limit = 0, 500
        done = False
        while not done:
            full_url = self.prep(data, url, timeseries, skip, limit, order, collapse, aggregate, transform)
            response = self.query(data=data, full_url=full_url, timeseries=timeseries)
            result = self.process(data, full_url, response, timeseries)
            skip += limit  # adding offset for next query
            final, done = self.done(final, result, skip)
            final = self.add_api_call(full_url, final)
        return final


class GetAllTimeseries(Query):
    def __init__(self, key, url=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        super().__init__(key, url, max_retries, retry_delay, **kwargs)
        self.name = "Get All Timeseries Metadata"
        self.key = "apiKey {}".format(key)
        self.log_prefix = self.build_log_prefix()

    @staticmethod
    def prep(url: str):
        return "{url}/timeseries".format(url=url)

    def post_process(self, full_url, result):
        if "data" in result:
            return {"data": result["data"]}
        return self.error(Error(check_name=self.log_prefix, value="No Timeseries Data", api_call=full_url))

    @defaults_from_attrs('url')
    def run(self, url=None, **kwargs):
        full_url = self.prep(url)
        response = self.query(data="", full_url=full_url)
        result = self.process(data="", full_url=full_url, response=response)
        result = self.post_process(full_url=full_url, result=result)
        result = self.add_api_call(full_url=full_url, result=result)
        return result  # data has key data


class SearchCompany(Query):
    def __init__(self, key, url=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        super().__init__(key, url, max_retries, retry_delay, **kwargs)
        self.name = "TimeseriesCompany"
        self.key = "apiKey {}".format(key)
        self.log_prefix = self.build_log_prefix()

    @staticmethod
    def prep(timeseries_id: str, url: str):
        if "error" in timeseries_id:
            return timeseries_id
        return "{url}/companies?name={timeseries_id}".format(url=url, timeseries_id=timeseries_id)

    @defaults_from_attrs('url')
    def run(self, data, url=None, **kwargs):
        full_url = self.prep(timeseries_id=data, url=url)
        response = self.query(data=data, full_url=full_url)
        result = self.process(data="", full_url=full_url, result=response, timeseries=data)
        result = self.add_api_call(full_url=full_url, result=result)
        return result



import requests
import logging
from datetime import timedelta
from prefect.utilities.tasks import defaults_from_attrs
from .query_service import Query


class QueryGSheet(Query):
    """
    Base class for querying CA
    """
    def __init__(self, key, url=None, a_range=None, sheet_id=None, sheet_name=None,
                 max_retries=3, retry_delay=timedelta(minutes=1)):
        super().__init__(key, url, max_retries, retry_delay)
        self.name = "Query GSheet"
        self.a_range = a_range
        self.sheet_id = sheet_id
        self.sheet_name = sheet_name
        self.log_prefix = self.build_log_prefix()

    @staticmethod
    def prep(url, a_range, sheet_id, sheet_name):
        url = "{url}/{sheet_id}/values/{sheet_name}!{a_range}".format(
            url=url, sheet_id=sheet_id, sheet_name=sheet_name, a_range=a_range)
        return url

    def query(self, url):
        logging.debug("Querying %s", url)
        url += "?key={key}".format(key=self.key)
        return requests.get(url, params={}, headers={})

    def post_process(self, result, collapse=True):
        companies = []
        if "values" not in result:
            return []
        values = result["values"]
        if not collapse:  # return early & don't collapse lists
            return values
        for company in values:
            companies += company
        logging.debug("{}: {} companies returned".format(self.log_prefix, len(companies)))
        companies = [x for x in companies if x != ""]
        return companies

    @defaults_from_attrs('url', 'a_range', 'sheet_id', 'sheet_name')
    def run(self, url=None, a_range=None, sheet_id=None, sheet_name=None, collapse=True):
        full_url = self.prep(url, a_range, sheet_id, sheet_name)
        response = self.query(full_url)
        result = self.process("", full_url, response)
        ids = self.post_process(result, collapse)
        return ids


class MapIDs(QueryGSheet):
    def __init__(self, key, url=None, a_range=None, sheet_id=None, sheet_name=None, key_idx=None, value_idx=None,
                 max_retries=3, retry_delay=timedelta(minutes=1)):
        super().__init__(key, url, a_range, sheet_id, sheet_name, max_retries, retry_delay)
        self.name = "Map IDS"
        self.key_idx = key_idx
        self.value_idx = value_idx

    @staticmethod
    def create_map(values, key_idx, value_idx) -> dict:
        maps = {}
        if len(values) < 2:
            logging.error("No maps %s", values)
            return maps
        for lst in values[1:]:
            if len(lst) < key_idx or len(lst) < value_idx:
                break
            maps[lst[key_idx]] = lst[value_idx]  # NOTE: hardcoded location of mapping
        return maps

    def mapped(self, ids, result, key_idx, value_idx):
        if "values" not in result:
            return []
        id_map = self.create_map(result["values"], key_idx, value_idx)
        try:
            return [id_map[x] for x in ids]
        except KeyError as err:
            logging.error("Key Error %s", err)
            return []

    @defaults_from_attrs('url', 'a_range', 'sheet_id', 'sheet_name', 'key_idx', 'value_idx')
    def run(self, ids, url=None, a_range=None, sheet_id=None, sheet_name=None, key_idx=None, value_idx=None):
        full_url = self.prep(url, a_range, sheet_id, sheet_name)
        response = self.query(full_url)
        result = self.process("", full_url, response)
        ids = self.mapped(ids, result, key_idx, value_idx)
        return ids



from distutils.core import setup

setup(
    name='commontaskz',
    packages=['commontaskz'],
    version='0.5.14',
    license='MIT',
    description='Common Prefect Tasks to query systems & process results',
    author='Aliza Rayman',  # Type in your name
    author_email='aliza.rayman@zirra.com',  # Type in your E-Mail
    url='https://github.com/zirra-com/commontaskz',  # Provide either the link to your github or to your website
    download_url='https://github.com/zirra-com/commontaskz/archive/v0.5.14.tar.gz',  # I explain this later on
    keywords=['TASKS'],  # Keywords that define your package best
    install_requires=[  # I get to this in a second
        'slackclient',
        'prefect',
        'datetime',
        'requests',
        'lz4'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)


import slack
import logging

from datamodelz.error import Error
from prefect import Task
from datetime import date, timedelta, datetime
from prefect.utilities.tasks import defaults_from_attrs


def get_field(dct: dict, name):
    """
    Will check & get field via name
    :param dct: dict
    :param name: any key stored in a dictionary
    :return: any value stored in a dictionary
    """
    if type(dct) == dict and name in dct:
        return dct[name]
    return None


def get_errors(errors, i) -> list:
    error_list = get_field(errors[i], "error")
    if type(error_list) == str:
        return [error_list]
    return error_list


def get_service_id(service_ids, i) -> str:
    service_id = ""
    if len(service_ids) > i and "error" not in service_ids[i]:
        service_id = service_ids[i]
    return service_id


class MakeErrorTask(Task):
    def __init__(self, token, titles=None, max_retries=3, retry_delay=timedelta(minutes=1), **kwargs):
        """
        :param titles:
        :param key: Slack Token to be used in slack.WebClient()
        """
        super().__init__(max_retries, retry_delay, **kwargs)
        self.name = "Make Error"
        self.titles = titles
        self.slack_client = slack.WebClient(token=token)

        # set automatically
        self.problems = []
        self.date = str(date.today())
        self.time = str(datetime.now().time())
        self.file = "results.csv"
        self.file_header = "Central Authority ID, Service ID, Errors"
        self.length = 0

    def make_file_name(self, titles: list) -> None:
        """
        Creates file name ex: datasys_kelvin_set_20190803.csv
        :param titles: list of strings
        :return: None
        """
        if titles:
            self.titles = titles
        title = "_".join(self.titles).lower().replace(" ", "_")
        self.file = "{}_{}.csv".format(title, self.date.replace("-", ""))
        return

    def success_alert(self) -> bool:
        """
        :return: bool: ran ok
        """
        response = self.slack_client.chat_postMessage(
            channel='#prefect-data-alerts',
            text=":smile: New Alert from `{}` there are {} errors (out of {})"
                .format(" ".join(self.titles), len(self.problems), self.length),
        )
        return response["ok"]

    def failure_alert(self) -> bool:
        """
        :return: bool: ran ok
        """
        response = self.slack_client.files_upload(
            channels='#prefect-data-alerts',
            file=self.file,
            filename=self.file,
            filetype="csv",
            initial_comment=":frowning: New Alert from `{}` there are {} errors (out of {})"
                .format(" ".join(self.titles), len(self.problems), self.length),
            title=self.file
        )
        return response["ok"]


class MakePivotErrorTask(MakeErrorTask):
    def __init__(self,
                 token,
                 titles=None,
                 max_retries=3,
                 retry_delay=timedelta(minutes=1),
                 header="Timeseries Code, Company, Check, Date, Value, Business Rule, Reference Link, API Call",
                 **kwargs):
        """
        needs to generate excel with 2 tabs and post to slack

        :param titles:
        :param key: Slack Token to be used in slack.WebClient()
        """
        super().__init__(token, titles, max_retries, retry_delay, **kwargs)
        self.name = "Make Pivot Error"
        self.header = header + "\n"
        self.counts = dict()
        self.formatted_counts = ""

    @staticmethod
    def writeable_list(lst, offset=False):
        if not lst:
            return ""
        writeable = ",".join(lst) + '\n'
        if offset:
            writeable = "," + writeable
        return writeable

    @staticmethod
    def flatten_errors(errors: list):
        error_all = []
        for dct_lst in errors:  # per timeseries
            for dct in dct_lst:   # per check list
                err_lst = get_field(dct, "error")
                if err_lst is None or not err_lst or type(err_lst) == str:
                    logging.debug(err_lst)  # TODO add
                    continue
                error_all += err_lst
        return error_all

    def error_ok(self, errors):
        if errors is None or type(errors) not in [str, list, dict]:
            logging.debug("Returning without making error list")
            return False
        return True

    def make_error_list(self, errors: list, flatten=False) -> None:
        # error has type Error
        if not errors:
            return
        self.length = len(errors)
        if flatten:
            if type(errors[0]) == dict:
                errors = [errors]
            errors = self.flatten_errors(errors)
        else:
            errors = get_field(errors, "error")

        for err in errors:
            if err is None:
                continue
            if type(err) == str:  # TODO:fix
                logging.error(err)
                self.problems.append(err)
                continue
            if err.empty():
                continue
            self.problems.append(err.excel_format())
        return

    def make_error_file(self, titles, collapse, aggregate) -> None:
        """
        requires: self.file, self.file_header, self.problems
        :return: None
        """
        if not self.problems:
            return
        with open(self.file, "w+") as csv:
            csv.write("Date, Time, Set Description, Collapse, Aggregate\n")
            csv.write(self.writeable_list([self.date, self.time, ' '.join(titles), collapse, aggregate, '\n']))
            csv.write(self.header)
            for lst in self.problems:
                try:
                    csv.write(lst)
                    csv.write('\n')
                except:
                    logging.error("cannot print problems list {}".format(lst))
                    continue
        return

    @defaults_from_attrs('titles')
    def run(self, titles=None, errors=None, collapse="", aggregate="", flatten=False, **kwargs) -> bool:
        """
        :param flatten:
        :param aggregate:
        :param collapse:
        :param titles:
        :param errors: list of strings
        :return: bool
        """
        if errors is None or type(errors) not in [list, dict]:
            errors = list()
        self.make_file_name(titles)
        self.make_error_list(errors, flatten)
        if self.problems:
            self.make_error_file(titles, collapse, aggregate)
            self.failure_alert()
            return False
        self.success_alert()
        return True

